"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var container_service_1 = require("../container-service");
var MODULE_META_DATA = 'react.di-module';
var Module = /** @class */ (function () {
    function Module(target, options) {
        this.target = target;
        this.options = options;
    }
    Module.prototype.getExternalContainer = function (context) {
        var container;
        if (this.isProviderContext(context)) {
            container = context.getContainer(this.target);
        }
        if (!container) {
            container = container_service_1.createContainerWithBindings(this.options);
            if (this.isProviderContext(context)) {
                context.addContainer(this.target, container);
            }
        }
        return container;
    };
    Module.prototype.getInternalContainer = function (context) {
        var container = this.getExternalContainer(context);
        if (this.options.imports) {
            if (!this.isProviderContext(context)) {
                throw new Error("Imports not supported here: When using imports with " + this.target.name +
                    " this module need to be nested in another module.");
            }
            container = this.getContainerWithImports(this.options.imports, container, context);
        }
        return container;
    };
    Module.prototype.isProviderContext = function (context) {
        return context && context.addContainer && context.getContainer;
    };
    Module.prototype.getContainerWithImports = function (imports, container, context) {
        return imports.reduce(function (container, toImportTarget) {
            var module = Module.getModule(toImportTarget);
            var toImportContainer = module.getExternalContainer(context);
            return container_service_1.merge(container, toImportContainer);
        }, container);
    };
    Module.getModule = function (target) {
        return Reflect.getMetadata(MODULE_META_DATA, target);
    };
    Module.setModule = function (target, moduleMeta) {
        Reflect.defineMetadata(MODULE_META_DATA, moduleMeta, target);
    };
    return Module;
}());
exports.Module = Module;
//# sourceMappingURL=Module.js.map